package ccl.servlet;

import ccl.util.Exitable;
import ccl.util.Init;
import ccl.util.Util;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * The debug server is a simple socket server
 * that prints each incoming request to standard
 * output as well as works as an echo server,
 * just like in the Java Tutorial example.
 * Its intended purposes is to quickly see
 * HTTP GET or POST requests to learn
 * and try something about this or other
 * network protocols.<p>
 * 
 * The user has to Ctrl-c to exit the process.
 *
 * The default port is 4444.
 * 
 * @author  Chr. Clemens Lee, clemens@kclee.com
 * @version $Id: DebugServer.java,v 1.3 2002/05/20 12:09:12 clemens Exp clemens $
 */
public class DebugServer implements Exitable
                                    , Runnable
{
    public static final int DEFAULT_PORT = 4444;

    /**
     * Specifies the legal command line options.
     */
    private static final String S_INI_FILE_CONTENT = "[Init]\n"
        + "Author=Chr. Clemens Lee\n"
        + "\n"
        + "[Help]\n"
        + "HelpUsage=\n"
        + "Options=\n";

    /**
     * Used by Exitable interface.
     */
    private boolean _bExit = false;

    /**
     * The Init object encapsulates all application initialization data.     */
    private Init _init = null;

    /**
     * Implements 'ccl.util.Exitable' interface.
     */
    public void setExit()
    {
        _bExit = true;
    }

    /**
     * Create this object and invoke the run method
     * if you want to start the server out of another object.
     */
    public DebugServer()
    {
        super();
    }

    /**
     * This method is invoked to setup the application.
     */
    public void initialize( String[] asArg, String sRCSHeader )
    {
        _init = new Init( this
                          , asArg
                          , sRCSHeader
                          , S_INI_FILE_CONTENT );
    }

    /**
     * This method contains the main batch logic.
     * Be careful to expect system exits if anything
     * goes wrong. Also, this method will never finish
     * by itself if no exception happens.
     */
    public void run()
    {
        ServerSocket serverSocket = null;

        try 
        {
            serverSocket = new ServerSocket( DEFAULT_PORT );
        } 
        catch ( IOException ioException ) 
        {
            Util.printlnErr( "Could not listen on port: " + DEFAULT_PORT + "." );

            // not very clean to exit here
            System.exit(1);
        }

        Util.println( "DebugServer started. Waiting for clients to connect on port " + DEFAULT_PORT + " ...\n" );

        while( true )
        {
            //
            // our client
            //

            Socket clientSocket = null;
            try 
{
                clientSocket = serverSocket.accept();
            }
 catch (IOException e) 
{
                Util.printlnErr("Accept failed.");

                // again not very clean to exit here
                System.exit(1);
            }
            
            Util.println( "New connection ...\n" );

            try
            {
                PrintWriter    out = new PrintWriter(clientSocket.getOutputStream(), true);
                BufferedReader in  = new BufferedReader
                       ( new InputStreamReader
                         ( clientSocket.getInputStream() ) );
                String inputLine  = "";
                String outputLine = "";
                
                while ((inputLine = in.readLine()) != null) 
                {
                    Util.println( inputLine );
                    out.println( inputLine );
                }

                out.close();
                in.close();
                clientSocket.close();
            }
            catch( IOException ioException )
            {
                Util.printlnErr( ioException );
            }

            Util.println( "\nClient connection closed, waiting for new connection ...\n" );
        }

        // this can never happen, so why close. I know, not very nice.
        // serverSocket.close();
    }

    public static void main( String[] asArg ) 
{
        DebugServer debugServer = new DebugServer();
        debugServer.initialize( asArg, "$Header: /home/clemens/src/java/ccl/src/ccl/servlet/RCS/DebugServer.java,v 1.3 2002/05/20 12:09:12 clemens Exp clemens $" );
        if ( debugServer._bExit ) 
{
            System.exit( 1 );
        }

        debugServer.run();

        // if run failed it did already exit

        System.exit( 0 );
    }
}
