/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is the reusable ccl library
 * (http://www.kclee.com/clemens/java/ccl/).
 *
 * The Initial Developer of the Original Code is Chr. Clemens Lee.
 * Portions created by Chr. Clemens Lee are Copyright (C) 1999
 * Chr. Clemens Lee. All Rights Reserved.
 *
 * Contributor(s): Chr. Clemens Lee <clemens@kclee.com>
 */

package ccl.util;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

/**
 * Framework and base class for all tests.
 * Inherit _doit in your test and use bugIf( boolean )
 * statements for your actual test comparisons.<p>
 *
 * Features:<br>
 * - support for stdout and stderr redirecting<p>
 *
 * Stdout redirection<p>
 *
 * Sometimes you test code which prints something to
 * the console while for the test you don't want to
 * confuse the user, since the test wants to print
 * status information as well.<p>
 * 
 * Console output will be redirected into two byte streams
 * when desired (use 'Test.redirectStandardStreams( true )').
 * The test output will still be printed to the normal
 * standard output streams.<p>
 *
 * When redirection takes place, the test can get a string
 * with the last output to these streams and use this
 * information for testing purposes as well. This buffer
 * can also be cleaned anytime on purpose.
 * <p>
 * 
 * For an example how to write your own tests take a look at
 * the ccl.util.test.UtilTest class
 *
 * @see ccl.util.test.UtilTest
 *
 * @version 1999-09-03 $Id: Test.java 1.21 2002/05/20 12:09:50 clemens Exp clemens $
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         &lt;<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>>
 */
public abstract class Test implements Runnable 
{
    private long _lGLobalTests = 0;
    private long _lLocalTests = 0;
    private long _lBugs = 0;
    private boolean _bTiming          = false;
    private boolean _bShowLiveSignals = false;
    private boolean _bLastSuccessfull = false;
    private boolean _bLastSubtest     = false;
    private int _subtests = 0;
    private long _lLastBugs = 0;
    private long _lLastLocalTests = 0;
    private String _sLastSubTest = null;
    private Vector _vCalendars = new Vector();

    private static Object _oValue = null;

    private void _showLiveSignal() 
    {
        if ( isVerbose() ) 
        {
            _ensureStandardStreams();
            if ( _bLastSubtest ) 
            {
                Util.println();
            }
            if ( _bLastSubtest || !_bLastSuccessfull ) 
            {
                if ( _subtests > 0 ) 
                {
                    Util.print( Util.getSpaces( (_subtests - 1 ) * 3 ) );
                }
            }
            Util.showLiveSignal();
            _ensureRedirectedStreams();
            _bLastSuccessfull = true;
            _bLastSubtest = false;
        }
    }

    protected Object _getValue() 
    {
        return _oValue;
    }

    /**
     * @param sName_ e.g. "jacob".
     */
    protected void _enterSubTest( String sName_ ) 
    {
        if ( !isVerbose() ) 
        {
            return;
        }

        _ensureStandardStreams();

        if ( _subtests > 0 ) 
        {
            Util.println();
            for( int subtest = 1; subtest < _subtests; subtest++ ) 
            {
                Util.print( "   " );
            }
            Util.print( "-> " );
        }
        else if ( 0 < _lLocalTests ) 
        {
            Util.println();
        }


        _sLastSubTest = sName_;
        _lLastBugs = _lBugs;
        Util.print( "Testing " +
                    sName_ + " " );
        _bLastSubtest = false;
        _bLastSuccessfull = true;

        if ( isTiming() ) 
        {
            _vCalendars.addElement( Util.getCalendar( "UTC" ) );
        }

        _ensureRedirectedStreams();

        _subtests++;
    }
    
    protected void _exitSubTest() 
    {
        if ( !isVerbose() ) 
        {
            return;
        }
        _subtests--;
        _lLastLocalTests = _lLocalTests;

        _ensureStandardStreams();

        if ( _bLastSubtest || !_bLastSuccessfull ) 
        {
            Util.println();
            if ( _subtests > 0 ) 
            {
                Util.print( Util.getSpaces( _subtests * 3 ) );
            }
        }
        else 
        {
            Util.print( " " );
        }
        if ( _lBugs == _lLastBugs )
        {
            Util.print( "SUCCESS" );
        }
        else 
        {
            Util.print( "FAILURE" );
        }
        if ( isTiming() ) 
        {
            Calendar calStart = (Calendar) _vCalendars.
                   lastElement();
            Calendar calEnd = Util.getCalendar( "UTC" );
            long delta = calEnd.getTime().getTime() - 
                   calStart.getTime().getTime();
            Util.debug( this, "_exitSubTest().delta: " + delta );
            Calendar calDelta = Util.getCalendar( "UTC" );
            calDelta.set( Calendar.MILLISECOND, (int) (delta                      % 1000) );
            calDelta.set( Calendar.SECOND     , (int) ((delta / 1000)             % 60  ) );
            calDelta.set( Calendar.MINUTE     , (int) ((delta / (60 * 1000))      % 60  ) );
            calDelta.set( Calendar.HOUR_OF_DAY, (int) ((delta / (60 * 60 * 1000)) % 24  ) );

            calEnd.setTime( new Date( delta ) );
            _vCalendars.removeElementAt( _vCalendars.size() - 1 );

            Util.print( " " + Util.getTime( calDelta ) + "." +
                        Util.paddWithZero( delta % 1000, 3 ) );
        }

        _ensureRedirectedStreams();

        _bLastSubtest = true;
    }

    /**
     * @deprecated   Use setTests(..) instead.
     */
    protected void _setTests(Test pTest_) 
    {
        setTests( pTest_ );
    }

    protected void _increment() 
    {
        _lGLobalTests++;
        _lLocalTests++;
        _showLiveSignal();
    }

    /**
     * @deprecated   Use setVerbose(..) instead.
     */
    protected void _showLiveSignals(boolean bShowLiveSignals_) 
    {
        _bShowLiveSignals = bShowLiveSignals_;
    }
    
    public long getLocalTests() 
    {
        return _lLocalTests;
    }
    
    public long getGlobalTests() 
    {
        return _lGLobalTests;
    }

    public long getBugs() 
    {
        return _lBugs;
    }
    
    public void setBug() 
    {
        _lBugs++;
    }
    
    public void setValue(Object oValue_) 
    {
        _oValue = oValue_;
    }

    public boolean isVerbose() 
    {
        return _bShowLiveSignals;
    }

    public void setVerbose( boolean bVerbose_ ) 
    {
        _bShowLiveSignals = bVerbose_;
    }

    /**
     * You want timing information? Set this to true!
     */
    public void setTiming( boolean bTiming_ ) 
    {
        _bTiming = bTiming_;
        if ( isTiming() ) 
        {
            setVerbose( isTiming() );
        }
    }

    public boolean isTiming() 
    {
        return _bTiming;
    }

    public static void printResult(Test pTest_) 
    {
        _ensureStandardStreams();

        Util.println("\n*   *   *   *   *   *   *   *   *   *   *   *   *   *");
        Util.println(pTest_.getClass().getName() + " has finished.");
        if (pTest_.getBugs() == 0) 
        {
            Util.println("Hey, kewl, all " + pTest_.getGlobalTests() +
                         " tests succeeded! :-)");
        }
        else 
        {
            Util.print( "I hate to say it, but " + pTest_.getBugs() +
                        " test" );
            if ( pTest_.getBugs() > 1 ) 
            {
                Util.print( "s" );
            }
            Util.println( " out of " + pTest_.getGlobalTests() +
                          " didn't succeed! :-(");
        }
    }

    private void _processFailure() 
    {
        this.setBug();
        _ensureStandardStreams();
        if ( isVerbose() && _bLastSuccessfull ) 
        {
            Util.println();
        }
        Util.print(this.getClass().getName() + ": ");
        if (_lLocalTests != _lGLobalTests) 
        {
            Util.println("Test " + _lLocalTests + "/" + _lGLobalTests +
                         " failed!");
        }
        else 
        {
            Util.println("Test " + _lLocalTests + " failed!");
        }
        _ensureRedirectedStreams();
    }

    private boolean _check( boolean bCondition_ ) 
    {
        _increment();
        _bLastSubtest = false;

        return bCondition_;
    }

    private void _printStackTrace() 
    {
        _ensureStandardStreams();
        Util.print( "    Stack trace: " );
        try 
        {
            throw new Exception();
        }
        catch( Exception exception ) 
        {
            exception.printStackTrace();
        }
        _ensureRedirectedStreams();
    }

    private void _printMessage( String sDescription ) 
    {
        Vector vLines = Util.stringToLines(sDescription);
        _ensureStandardStreams();
        for(Enumeration e = vLines.elements(); e.hasMoreElements(); ) 
        {
            Util.println("    " + ((String) e.nextElement()));
        }
        _ensureRedirectedStreams();
    }

    private static boolean _bTest = false;
    
    public Test() 
    {
        super();
    }
    
    public Test( Test tstParent_ ) 
    {
        super();
        
        //_bTest = true;
        //_setTests(pTest_);     
        setVerbose( tstParent_.isVerbose() );
        setTiming ( tstParent_.isTiming () );

        _subtests = tstParent_._subtests;
    }
    
    public boolean bugIf(boolean bCondition) 
    {
        if ( _check( bCondition ) ) 
        {
            _processFailure();
            _printStackTrace();
        }
        _bLastSuccessfull = !bCondition;
        
        return bCondition;
    }

    public boolean bugIf(boolean bCondition, String sDescription) 
    {
        if ( _check(bCondition) ) 
        {
            _processFailure();
            _printMessage( sDescription );
            _printStackTrace();
        }
        _bLastSuccessfull = !bCondition;
        
        return bCondition;
    }
    
    public boolean bugIf( boolean bCondition, 
                          String sDescription,
                          Throwable pThrowable)
    {
        if ( _check(bCondition) ) 
        {
            _processFailure();
            _printMessage( sDescription );
            _ensureStandardStreams();
            Util.print( "    " );
            pThrowable.printStackTrace();
            _ensureRedirectedStreams();
        }
        _bLastSuccessfull = !bCondition;
        
        return bCondition;
    }

    /**
     * Conduct a check that a given object is not null.
     * Otherwise count as a failed test.
     */
    public void assertNotNull( Object object_ ) 
    {
        bugIf( object_ == null, "Assert not null test failed!" );
    }
    
    /**
     * Conduct a check that a given object is not null.
     * Otherwise count as a failed test.
     *
     * @param   sMessage_   The message gets printed when the
     *                      assertion fails.
     */
    public void assertNotNull( Object object_
                               , String sMessage_ )
    {
        bugIf( object_ == null, sMessage_ );
    }
    
    /**
     * Conduct a check that a given object fullfills a 
     * given condition.
     * If this condition is not fullfilled (boolean is false)
     * count this check as a failed test.
     */
    public void assert( boolean bCheck_ ) 
    {
        bugIf( bCheck_ == false, "Assert test failed!" );
    }
    
    /**
     * Conduct a check that a given object fullfills a 
     * given condition.
     * If this condition is not fullfilled (boolean is false)
     * count this check as a failed test.
     *
     * @param   sMessage_   The message gets printed when the
     *                      assertion fails.
     */
    public void assert( boolean bCheck_
                        , String sMessage_ )
    {
        bugIf( bCheck_ == false, sMessage_ );
    }
    
    /**
     * Inherit this method to do your test inside of it.
     *
     * @exception   Exception   Whatever can go wrong.
     */
    protected abstract void _doIt() throws Exception;
    
    public void run() 
    {
        if ( isVerbose() ) 
        {
            String sTestName = Util.getObjectName( this );
            int testIndex = sTestName.lastIndexOf( "Test" );
            if ( testIndex == -1 ) 
            {
                testIndex = sTestName.length();
            }
            _enterSubTest( sTestName.substring( 0, testIndex ) );
        }
        try 
        {
            // hier wird die arbeitet gemacht und
            // diese Fkt. soll auch berschrieben werden.
            _doIt();
        }
        catch( Throwable throwable ) 
        {
            bugIf( true , "Test error!", throwable );
        }
        if ( isVerbose() ) 
        {
            _exitSubTest();
        }
    }

    /**
     * example code for a real test main method<p>
     *
     * <pre>
     * Test pTest = (Test)(new SomeTest());
     * pTest.initialize( argv );
     * pTest.setVerbose( true );
     * pTest.run();
     * printResult( pTest );
     *
     * System.exit( 0 );
     * </pre>
     */
    public static void main(String[] argv) 
    {
        System.exit( 0 );
    }

    /**
     * Process the arguments from the main class here.
     * This way you can pass arguments to this test also
     * from the test center, which does not use the
     * main method to invoke a test.
     */
    public void initialize( String[] asArg_ ) 
    {
        Util.debug( this, "initialize(..).args: " +
                    Util.toString( Util.objectsToVector
                                   ( asArg_ ) ) );
    }

    public static boolean isTest() 
    {
        return _bTest;
    }

    public static Object getValue() 
    {
        return _oValue;
    }

    /**
     * Returns the directory of the class on which this method is
     * invoked.
     *
     * @see          #getTestDirectory()
     * @deprecated   Use getTestDirectory instead as this method
     *               will not work well together with jar files.
     */
    public String getTestClassDirectory() 
    {
        return FileUtil.getClassPath( this );
    }

    /**
     * This method returns the full path of a dedicated test directory
     * under the application directory.
     * 
     * @return   a string for a director with test data for the
     *           current application, 
     *           e.g. "/home/clemens/src/java/jacob/test".
     */
    public String getTestDirectory() 
    {
        return FileUtil.concatPath( ClassPathUtil.getApplicationHome( this )
                                    , "test" );
    }

    public String toString() 
    {
        /*return "Result for " + getClass().getName() + ":";*/
        return ">> no comment available <<";
    }

    public void setTests( Test test_ ) 
    {
        if (test_ == null) 
        {
            _lGLobalTests = 0;
            _lBugs = 0;
            _lLocalTests = 0;
        }
        else 
        {
            _lGLobalTests += test_.getGlobalTests();
            _lBugs += test_.getBugs();
            _bLastSubtest = true;
            //_lLocalTests++;
            //_showLiveSignal();
        }
    }

    /**
     * Return a one line comment for this test.
     * Should be overwritten be test classes inheriting
     * from this class.
     */
    public String getComment() 
    {
        return ">> no comment available <<";
    }

    private static boolean _bRedirectStandardStreams = false;

    private static PrintStream _oldOut = null;
    private static PrintStream _newOut = null;
    private static PrintStream _oldErr = null;
    private static PrintStream _newErr = null;

    /**
     * When we need to switch between two std output streams,
     * this method helps doing so. E.g. we redirect
     * std out but want to print a "test SUCCESS" message,
     * then we use _setOutOld, print, _setOutNew methods.
     */
    private static void _setOutNew() 
    {
        System.setOut( _newOut );
        System.setErr( _newErr );
    }

    /**
     * When we need to switch between two std output streams,
     * this method helps doing so. E.g. we redirect
     * std out but want to print a "test SUCCESS" message,
     * then we use _setOutOld, print, _setOutNew methods.
     */
    private static void _setOutOld() 
    {
        System.setOut( _oldOut );
        System.setErr( _oldErr );
    }

    /**
     * If standard streams are redirected, normal streams
     * are used now.
     */
    private static void _ensureStandardStreams() 
    {
        if ( _bRedirectStandardStreams ) 
        {
            _setOutOld();
        }
    }

    /**
     * If standard streams are redirected, ensure that
     * redirected streams are in place.
     */
    private static void _ensureRedirectedStreams() 
    {
        if ( _bRedirectStandardStreams ) 
        {
            _setOutNew();
        }
    }

    /**
     * If standard streams are redirected, redirection
     * is established again.
     */

    /**
     * Redirect stdout and stderr into private streams
     * so the do not confuse the user with test output.
     *
     * Look at the class documentation at the top for
     * detailed stdout redirection documentation.
     */
    public static void redirectStandardStreams( boolean bRedirect_ ) 
    {
        if ( _bRedirectStandardStreams == bRedirect_ ) 
        {
            return;
        }

        _bRedirectStandardStreams = bRedirect_;

        if ( _bRedirectStandardStreams ) 
        {
            if ( _newOut == null ) 
            {
                // redirect standard output
                _oldOut = System.out;
                ByteArrayOutputStream byteStream      = new ByteArrayOutputStream();
                _newOut = new PrintStream( byteStream );
            
                // redirect standard error
                _oldErr = System.err;
                ByteArrayOutputStream errorByteStream = new ByteArrayOutputStream();
                _newErr = new PrintStream( errorByteStream );
            }
            _setOutNew();
        }
        else 
        {
            _setOutOld();
        }
    }
}
