/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is the reusable ccl library
 * (http://www.kclee.com/clemens/java/ccl/).
 *
 * The Initial Developer of the Original Code is Chr. Clemens Lee.
 * Portions created by Chr. Clemens Lee are Copyright (C) 1999
 * Chr. Clemens Lee. All Rights Reserved.
 *
 * Contributor(s): Chr. Clemens Lee <clemens@kclee.com>
 */

package ccl.util.test;

import ccl.util.FileUtil;
import ccl.util.Test;
import ccl.util.Util;
import java.io.File;
import java.io.IOException;

/**
 * Tests for ccl.util.FileUtil.
 *
 * @version $Id: FileUtilTest.java,v 1.18 2002/05/20 12:09:55 clemens Exp clemens $
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         &lt;<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>>
 */
public class FileUtilTest extends Test 
{
    /**
     * Tests method 'FileUtil.getFiles(String,String)'.
     * Especially if empty file suffixes return all files
     * in a directory.
     *
     * @exception   IOException   if anything goes wrong.
     */
    private void _checkGetFiles() 
        throws IOException
    {
        String sTestDir = FileUtil.createTempDir();

        assert( FileUtil.getFiles( sTestDir, "" ).size() == 0 );

        FileUtil.writeFile( FileUtil.concatPath( sTestDir, ".emacs" )
                            , ";; test file\n" );
        FileUtil.writeFile( FileUtil.concatPath( sTestDir, "ccl.jar" )
                            , "test" );

        assert( FileUtil.getFiles( sTestDir, ".jar" ).size() == 1 );
        assert( FileUtil.getFiles( sTestDir, ".zip" + File.pathSeparator + ".jar" ).size() == 1 );
        assert( FileUtil.getFiles( sTestDir, ".jar" + File.pathSeparator + ".emacs" ).size() == 2 );

        int result = FileUtil.getFiles( sTestDir, "" ).size();
        assert( result == 2,
                "There should be 2 files found in the test directory but we got " + result + "." );

        FileUtil.deleteRecursively( sTestDir );
        bugIf( FileUtil.exists( sTestDir ) );
    }

    public FileUtilTest() 
    {
        super();
    }

    public FileUtilTest(Test pTest_) 
    {
        super(pTest_);
    }

    /**
     * @throws   Exception   Whatever can go wrong.
     */
    protected void _doIt() 
        throws Exception
    {
        String sLocalPath = FileUtil.getClassPath(this);
        
        // Nr. 1    fRnd
        String sContent = "bla bla Schwachsinn!!!\n";
        String sFileName = FileUtil.concatPath(sLocalPath, "test.txt");
        String sFileNameBak = sFileName + ".bak";
        FileUtil.delete(sFileName);
        bugIf(FileUtil.exists(sFileName));
        try 
        {
            FileUtil.writeFile(sFileName, sContent);
            bugIf(false);
        }
        catch(Exception e) 
        {
            bugIf(true);
        }
        bugIf(!FileUtil.exists(sFileName));
        // move test
        FileUtil.move(sFileName, sFileNameBak);
        bugIf(FileUtil.exists(sFileName));
        bugIf(!FileUtil.exists(sFileNameBak));
        FileUtil.delete(sFileNameBak);
        bugIf(FileUtil.exists(sFileNameBak));
        
        // Nr. 7
        try 
        {
            FileUtil.writeFile(sFileName, sContent);
            FileUtil.writeFile(sFileNameBak, sContent);
            bugIf(false);
        }
        catch(Exception e) 
        {
            bugIf(true);
        }
        bugIf(!FileUtil.exists(sFileName));
        bugIf(!FileUtil.exists(sFileNameBak));
        FileUtil.move(sFileName, sFileNameBak);
        bugIf(FileUtil.exists(sFileName));
        bugIf(!FileUtil.exists(sFileNameBak));
        FileUtil.delete(sFileNameBak);
        bugIf(FileUtil.exists(sFileNameBak));
        
        // Nr. 13
        // move and source does not exist
        try 
        {
            FileUtil.writeFile(sFileNameBak, sContent);
            bugIf(false);
        }
        catch(Exception e) 
        {
            bugIf(true);
        }
        bugIf(FileUtil.exists(sFileName));
        bugIf(!FileUtil.exists(sFileNameBak));
        boolean bError = FileUtil.move( sFileNameBak, sFileName );
        bugIf( bError, "Could not move file " + sFileNameBak + " to " + sFileName );
        bugIf( FileUtil.exists(sFileNameBak) );
        
        // Nr. 18   exists[File|Dir]
        bugIf(FileUtil.exists("_Asdf_"));
        bugIf(FileUtil.exists("h:\\eigen\\src\\java\\lib\\ccl\\util\\test\\_Asdf_"));
        bugIf( !FileUtil.exists( System.getProperty( "user.dir" ) ) );
        /*// Nr. 21
          bugIf(!FileUtil.exists(FileUtil.concatPath(sLocalPath, ".." + File.separator + "test")));
          bugIf(!FileUtil.exists(FileUtil.concatPath(sLocalPath, ".." + File.separator + "test" + File.separator)));*/
        // Nr. 21
        bugIf(FileUtil.exists("h:/eigen/src/java/lib/ccl/util/test/_Asdf_"));
        // Nr. 22   exists[File|Dir]
        bugIf(FileUtil.existsFile("_Asdf_"));
        bugIf(FileUtil.existsDir("_Asdf_"));
        bugIf(FileUtil.existsFile(sLocalPath));

        // concatPath test
        bugIf( Util.endsWith( "", File.separatorChar ) );
        String sConcatPathResult = ccl.util.FileUtil.concatPath( "", "gnudoit" );
        bugIf( !sConcatPathResult.equals( "gnudoit" ), 
               "sConcatPathResult: " + sConcatPathResult );

        // FileUtil.getPackagePath(..) test
        String sPackagePath = null;
        String sHomePath = System.getProperty( "user.home" );
        bugIf( !FileUtil.existsDir( sHomePath ) );
        bugIf( FileUtil.existsFile( sHomePath ) );
        try 
        {
            sPackagePath = FileUtil.getPackagePath
                   ( ".", sHomePath );
            bugIf( !sPackagePath.equals( sHomePath ), "sPackagePath: " +
                   sPackagePath );
        }
        catch( Exception pException ) 
        {
            bugIf( true, "pException: " + pException );
        }

        // test directory deletion
        String sTempDir = FileUtil.getTempDir();
        sTempDir = FileUtil.concatPath( sTempDir, "fileutiltest" );
        FileUtil.md( sTempDir );
        assert( FileUtil.existsDir( sTempDir ) );
        String sTempFile = FileUtil.concatPath( sTempDir
                                                , "file" );
        FileUtil.writeFile( sTempFile, "dummytext" );
        //Util.setDebug( true );
        assert( FileUtil.existsFile       ( sTempFile ) );
        bugIf ( FileUtil.deleteRecursively( sTempDir  ) );
        bugIf ( FileUtil.exists           ( sTempDir  ) );

        // check getResourceAsStream
        {
            String sIniContent = FileUtil.getResourceAsString
                   ( this, "clearsight.ini" );
            assert( sIniContent.indexOf( "\nAuthor=Chr. Clemens Lee\n" ) != -1
                    , "Resource string: " + sIniContent );
        }

        _checkGetFiles();
    }

    public static void main(String[] args) 
    {
        Test pTest = (Test) (new FileUtilTest());
        pTest.setVerbose( true );
        pTest.run();
        pTest.printResult(pTest);

        System.exit(0);
    }
}

/*class FileUtilTest$Write {
    public static void main( String[] asArg_ ) {
        String sTempFile = asArg_[ 0 ];

        try {
            Util.println( "\nsystem start" );
            FileWriter writer = new FileWriter( sTempFile );
            writer.write( "something\n" );
            Util.sleep( 1 );
            writer.write( "something\n" );
            Util.sleep( 10 );
            writer.write( "something\n" );
            writer.close();
        } catch( Exception exception ) {
            Util.printlnErr( exception );
        }
        Util.println( "\nsystem stop" );
    }
    }*/
